package ch.psi.daq.test.queryrest.controller;

import javax.annotation.Resource;

import org.junit.After;
import org.junit.Test;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultHandlers;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import ch.psi.daq.cassandra.util.test.CassandraDataGen;
import ch.psi.daq.common.ordering.Ordering;
import ch.psi.daq.query.model.AggregationType;
import ch.psi.daq.query.model.impl.PulseRangeQuery;
import ch.psi.daq.query.model.impl.TimeRangeQuery;
import ch.psi.daq.query.model.impl.TimeRangeQueryDate;
import ch.psi.daq.queryrest.controller.QueryRestController;
import ch.psi.daq.queryrest.filter.CorsFilter;
import ch.psi.daq.test.cassandra.admin.CassandraTestAdmin;
import ch.psi.daq.test.queryrest.AbstractDaqRestTest;

/**
 * Tests the {@link DaqController} implementation.
 */
public class QueryRestControllerTest extends AbstractDaqRestTest {

   @Resource
   private CassandraTestAdmin cassandraTestAdmin;

   @Resource
   private CassandraDataGen dataGen;

   @Resource
   private CorsFilter corsFilter;

   public static final String[] TEST_CHANNEL_NAMES = new String[] {"testChannel1", "testChannel2"};

   @After
   public void tearDown() throws Exception {}

   @Test
   public void testChannelNameQuery() throws Exception {

      this.mockMvc.perform(
            MockMvcRequestBuilders
                  .get(QueryRestController.CHANNELS)
                  .contentType(MediaType.APPLICATION_JSON))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").value("BooleanScalar"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1]").value("BooleanWaveform"));

   }

   @Test
   public void testSpecificChannelSearch() throws Exception {
      this.mockMvc.perform(
            MockMvcRequestBuilders
                  .get(QueryRestController.CHANNELS + "/integer")
                  .contentType(MediaType.APPLICATION_JSON))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").value("IntegerScalar"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1]").value("IntegerWaveform"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[2]").value("UIntegerScalar"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[3]").value("UIntegerWaveform"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[4]").doesNotExist());
   }

   @Test
   public void testCorsFilterNoHeaders() throws Exception {
      corsFilter.setForceAllHeaders(false);
      this.mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).addFilters(corsFilter).build();

      this.mockMvc.perform(
            MockMvcRequestBuilders
                  .options(QueryRestController.CHANNELS)
                  .contentType(MediaType.APPLICATION_JSON))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            // we didn't set the 'Origin' header so no access-control
            .andExpect(MockMvcResultMatchers.header().doesNotExist("Access-Control-Allow-Origin"));
   }

   @Test
   public void testCorsFilterIncludesHeaders() throws Exception {
      // all headers are set
      this.mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).addFilters(corsFilter).build();

      this.mockMvc.perform(
            MockMvcRequestBuilders
                  .options(QueryRestController.CHANNELS)
                  .header("Origin", "*")
                  .contentType(MediaType.APPLICATION_JSON))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            // we didn't set the 'Origin' header so no access-control
            .andExpect(MockMvcResultMatchers.header().string("Access-Control-Allow-Origin", "*"));

      this.mockMvc.perform(
            MockMvcRequestBuilders
                  .options(QueryRestController.CHANNELS)
                  .header("Origin", "http://localhost:8080")
                  .contentType(MediaType.APPLICATION_JSON))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            // we didn't set the 'Origin' header so no access-control
            .andExpect(MockMvcResultMatchers.header().string("Access-Control-Allow-Origin", "http://localhost:8080"));

      this.mockMvc.perform(
            MockMvcRequestBuilders
                  .options(QueryRestController.CHANNELS)
                  .header("Origin", "someBogusDomain.com")
                  .contentType(MediaType.APPLICATION_JSON))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.header().string("Access-Control-Allow-Origin", "*"));

   }

   @Test
   public void testCorsFilterMismatchSpecificOrigin() throws Exception {
      corsFilter.setForceAllHeaders(false);
      this.mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).addFilters(corsFilter).build();

      this.mockMvc
            .perform(
                  MockMvcRequestBuilders
                        .options(QueryRestController.CHANNELS)
                        .header("Origin", "*")
                        .header("Access-Control-Request-Method", "GET")
                        .contentType(MediaType.APPLICATION_JSON))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.header().string("Access-Control-Allow-Origin", "*"))
            .andExpect(
                  MockMvcResultMatchers.header().string("Access-Control-Allow-Headers",
                        "Origin, Authorization, Accept, Content-Type"));

      this.mockMvc
            .perform(
                  MockMvcRequestBuilders
                        .options(QueryRestController.CHANNELS)
                        .header("Origin", "someBogusDomain.com")
                        .header("Access-Control-Request-Method", "GET")
                        .contentType(MediaType.APPLICATION_JSON))
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.header().doesNotExist("Access-Control-Allow-Origin"))
            .andExpect(
                  MockMvcResultMatchers.header().string("Access-Control-Allow-Headers",
                        "Origin, Authorization, Accept, Content-Type"));
   }


   @Test
   public void testPulseRangeQuery() throws Exception {
      PulseRangeQuery request = new PulseRangeQuery(
            100,
            101,
            TEST_CHANNEL_NAMES);

      String content = mapper.writeValueAsString(request);
      System.out.println(content);

      content = "{\"channels\":[\"testChannel1\",\"testChannel2\"],\"startPulseId\":100,\"endPulseId\":101}";

      this.mockMvc
            .perform(MockMvcRequestBuilders
                  .post(QueryRestController.QUERY)
                  .contentType(MediaType.APPLICATION_JSON)
                  .content(content))

            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel").value("testChannel1"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].pulseId").value(101))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].channel").value("testChannel2"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[1].pulseId").value(101));
   }

   @Test
   public void testTimeRangeQuery() throws Exception {
      TimeRangeQuery request = new TimeRangeQuery(
            100,
            101,
            TEST_CHANNEL_NAMES);

      String content = mapper.writeValueAsString(request);

      this.mockMvc.perform(MockMvcRequestBuilders
            .post(QueryRestController.QUERY)
            .contentType(MediaType.APPLICATION_JSON)
            .content(content))

            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel").value("testChannel1"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].pulseId").value(101))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].channel").value("testChannel2"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[1].pulseId").value(101));
   }

   @Test
   public void testDateRangeQuery() throws Exception {
      String startDate = TimeRangeQueryDate.format(100);
      String endDate = TimeRangeQueryDate.format(101);
      TimeRangeQueryDate request = new TimeRangeQueryDate(
            startDate,
            endDate,
            TEST_CHANNEL_NAMES);

      String content = mapper.writeValueAsString(request);
      System.out.println(content);

      this.mockMvc
            .perform(
                  MockMvcRequestBuilders
                        .post(QueryRestController.QUERY)
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(content)
            )
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel").value("testChannel1"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data[1].pulseId").value(101))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].channel").value("testChannel2"))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[0].pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[1].data[1].pulseId").value(101));
   }

   @Test
   public void testExtremaAggregation() throws Exception {
      PulseRangeQuery request = new PulseRangeQuery(
            100,
            101,
            false,
            Ordering.asc,
            AggregationType.extrema,
            TEST_CHANNEL_NAMES[0]);

      String content = mapper.writeValueAsString(request);

      this.mockMvc
            .perform(MockMvcRequestBuilders.post(QueryRestController.QUERY)
                  .contentType(MediaType.APPLICATION_JSON)
                  .content(content))

            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())

            .andExpect(MockMvcResultMatchers.jsonPath("$").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel").value(TEST_CHANNEL_NAMES[0]))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.minima").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.minima.min").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.minima.min.value").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.minima.min.value").value(Double.valueOf(100)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.minima.min.event").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.minima.min.event.pulseId").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.minima.min.event.pulseId").value(100))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.maxima").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.maxima.max").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.maxima.max.value").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.maxima.max.value").value(Double.valueOf(101)))
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.maxima.max.event").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.maxima.max.event.pulseId").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].data.maxima.max.event.pulseId").value(101));
   }


}
