/**
 * 
 */
package ch.psi.daq.test.queryrest.query;

import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.apache.commons.lang.NotImplementedException;

import ch.psi.daq.cassandra.reader.CassandraReader;
import ch.psi.daq.cassandra.reader.query.PulseIdRangeQuery;
import ch.psi.daq.cassandra.reader.query.TimeRangeQuery;
import ch.psi.daq.cassandra.util.test.CassandraDataGen;
import ch.psi.daq.common.ordering.Ordering;
import ch.psi.daq.domain.DataEvent;
import ch.psi.daq.domain.cassandra.ChannelEvent;
import ch.psi.daq.domain.cassandra.MetaPulseId;
import ch.psi.daq.domain.cassandra.MetaTime;
import ch.psi.daq.domain.cassandra.querying.ChannelEventQuery;
import ch.psi.daq.domain.cassandra.querying.ChannelEventQueryInfo;
import ch.psi.daq.domain.cassandra.querying.EventQuery;

import com.google.common.collect.Lists;

/**
 * @author zellweger_c
 *
 */
public class DummyCassandraReader implements CassandraReader {
   
   private static final int KEYSPACE = 1;
   private CassandraDataGen dataGen;
   private String[] channels;
   

   /**
    * 
    */
   public DummyCassandraReader() {
      this.dataGen = new CassandraDataGen();
      this.channels = new String[]{
            "testChannel1", 
            "testChannel2",
            "BooleanScalar",
            "BooleanWaveform",
            "ByteScalar",
            "ByteWaveform",
            "DoubleScalar",
            "DoubleWaveform",
            "FloatScalar",
            "FloatWaveform",
            "IntegerScalar",
            "IntegerWaveform",
            "LongScalar",
            "LongWaveform",
            "ShortScalar",
            "ShortWaveform",
            "StringScalar",
            "UByteScalar",
            "UByteWaveform",
            "UIntegerScalar",
            "UIntegerWaveform",
            "ULongScalar",
            "ULongWaveform",
            "UShortScalar",
            "UShortWaveform"};
   }
   /**
    * @{inheritDoc}
    */
   @Override
   public Stream<String> getChannelStream(String regex) {
      Stream<String> channelStream = Stream.of(channels);
      if (regex != null) {
         Pattern pattern = Pattern.compile(regex.toLowerCase());
         channelStream = Stream.of(channels).filter(channel -> pattern.matcher(channel.toLowerCase()).find());
      }
      return channelStream;
   }

   /**
    * @{inheritDoc}
    */
   @Override
   public Stream<? extends DataEvent> getEventStream(String channel, long startPulseId, long endPulseId,
         Ordering ordering, String... columns) {
      return getDummyEventStream(channel, startPulseId, endPulseId);
   }

   /**
    * @{inheritDoc}
    */
   @Override
   public Stream<? extends DataEvent> getEventStream(String channel, long startMillis, long startNanos, long endMillis,
         long endNanos, Ordering ordering, String... columns) {
      return getDummyEventStream(channel, startMillis, endMillis);
   }
   
   /**
    * @{inheritDoc}
    */
   @Override
   public Stream<ChannelEvent> getEventStream(EventQuery eventQuery, Stream<? extends ChannelEventQuery> queryProviders) {
      List<ChannelEvent> result = Lists.newArrayList();
      queryProviders.forEach(ceq -> {
         if (ceq instanceof ChannelEventQueryInfo) {
            result.add(getEvent((ChannelEventQueryInfo) ceq));
         } else {
            throw new NotImplementedException("This is not yet implemented!");
         }
      });
      return result.stream();
   }
   /**
   * @{inheritDoc}
   */
   @Override
   public Stream<ChannelEvent> getEventStream(PulseIdRangeQuery query) {
      Stream<ChannelEvent> dummyEventStream = getDummyEventStream(query.getChannel(), query.getStartPulseId(), query.getEndPulseId())
            .map(ce -> { return (ChannelEvent) ce; });
      return dummyEventStream;
      
   }
   /**
    * @{inheritDoc}
    */
   @Override
   public Stream<ChannelEvent> getEventStream(TimeRangeQuery query) {
      Stream<ChannelEvent> dummyEventStream = getDummyEventStream(query.getChannel(), query.getStartMillis(), query.getEndMillis())
            .map(ce -> { return (ChannelEvent) ce; });
      return dummyEventStream;
   }
   
   
   private Stream<? extends DataEvent> getDummyEventStream(String channel, long startIndex, long endIndex) {
      
      return dataGen.generateData(startIndex, (endIndex-startIndex + 1),
            i -> i,
            i -> i,
            i -> i,
            i -> i,
            i -> i,
            i -> Long.valueOf(i),
            channel).stream();
   }

   private List<? extends DataEvent> getDummyEvents(String channel, long startIndex, long endIndex) {
      
      return dataGen.generateData(startIndex, (endIndex-startIndex + 1),
            i -> i,
            i -> i,
            i -> i,
            i -> i,
            i -> i,
            i -> Long.valueOf(i),
            channel);
   }

   /**
    * @{inheritDoc}
    */
   @Override
   public List<String> getChannels() {
      return Lists.newArrayList(channels);
   }

   /**
    * @{inheritDoc}
    */
   @Override
   public List<String> getChannels(String regex) {
      return Lists.newArrayList(channels).stream().filter(s -> { return s.contains(regex); }).collect(Collectors.toList());
   }

   /**
    * @{inheritDoc}
    */
   @Override
   public ChannelEvent getEvent(ChannelEventQueryInfo queryInfo, String... columns) {
      if (queryInfo.getPulseId() > 0) {
         return (ChannelEvent) getDummyEvents(queryInfo.getChannel(), queryInfo.getPulseId(), queryInfo.getPulseId()).get(0);
      }
      return (ChannelEvent) getDummyEvents(queryInfo.getChannel(), queryInfo.getGlobalMillis(), queryInfo.getGlobalMillis()).get(0);
   }

   /**
    * @{inheritDoc}
    */
   @Override
   public CompletableFuture<ChannelEvent> getEventAsync(ChannelEventQueryInfo queryInfo, String... columns) {
      throw new NotImplementedException();
   }

   /**
    * @{inheritDoc}
    */
   @Override
   public Stream<? extends ChannelEventQuery> getChannelEventQueryStream(PulseIdRangeQuery query) {
      
      
      return dataGen.generateMetaPulseId(
            KEYSPACE, 
            query.getStartPulseId(), 
            (query.getEndPulseId() - query.getStartPulseId() + 1),
            i -> i,
            i -> i,
            i -> i,
            query.getChannel()).stream();
   }

   /**
    * @{inheritDoc}
    */
   @Override
   public Stream<? extends ChannelEventQuery> getChannelEventQueryStream(TimeRangeQuery query) {
      
      
      return dataGen.generateMetaTime(
            KEYSPACE, 
            query.getStartMillis(), 
            (query.getEndMillis() - query.getStartMillis() + 1), 
            i -> i,
            i -> i,
            i -> i,
            query.getChannel()).stream();
   }
   
   /**
    * @{inheritDoc}
    */
   @Override
   public Stream<MetaPulseId> getMetaPulseIdStream(PulseIdRangeQuery query) {
      
      return getChannelEventQueryStream(query).map(r -> { return (MetaPulseId) r; });
      
   }


   /**
    * @{inheritDoc}
    */
   @Override
   public Stream<MetaTime> getMetaTimeStream(TimeRangeQuery query) {
      
      return getChannelEventQueryStream(query).map(r -> { return (MetaTime) r; });
   }

}
